/**
 * apeman app to recover password.
 * @function apemanAppRecover
 * @param {object} UserRecover - RecoverModel.
 * @param {object} [options] - Optional settings.
 * @param {number} [options.expireDuration=ONE_DAY] - Expire duration
 * @param {function} [options.beforeSend=noop()] - Handle before send
 * @param {function} [options.beforeReset=noop()] - Handle before reset
 * @param {function} [options.afterSend=noop()] - Handle after send
 * @param {function} [options.afterReset=noop()] - Handle after reset
 * @param {object} [options.sendSchema=defaultSchema('send')] - Schema for send.
 * @param {object} [options.resetSchema=defaultSchema('send')] - Schema for reset.
 * @param {number} [options.passwordSpec='pdkdf2:120:84'] - Password algorithm, iterations, length.
 * @returns {function} - Defined app function.
 *
 */

'use strict'

const assert = require('assert')
const defineDigest = require('apeman-app-sign/lib/digesting/define_digest')
const rest = require('apeman-app-rest')
const schemas = require('./schemas')

const debug = require('debug')('apeman:app:recover')

const ONE_DAY = 24 * 60 * 60 * 1000

/** @lends create */
function create (UserRecover, options = {}) {
  assert.ok(UserRecover, 'UserRecover is required.')
  assert.ok(!!(UserRecover.$$inherited && UserRecover.$$inherited.ApUserRecover), 'UserRecover should inherit ApUserRecover.')

  let pathname = options.pathname || '/recover'
  let format = options.format
  let passwordSpec = options.passwordSpec
  let digest = defineDigest(passwordSpec)
  let expireDuration = options.expireDuration || ONE_DAY
  debug(`Password spec: ${passwordSpec}`)
  return rest(UserRecover, {
    format,
    pathname,
    createSchema: options.createSchema || schemas('recover-create'),
    updateSchema: options.updateSchema || schemas('recover-update'),
    bulkUpdateDisabled: true,
    bulkDestroyDisabled: true,
    relatedDisabled: true,
    relationDisabled: true,
    doCreate: require('./doing/do_create_recover')({ expireDuration }),
    doUpdate: require('./doing/do_update_recover')({ digest })
  })
}

module.exports = create
