/**
 * @function doUpdateRecover
 * @param {Object} config - Configuration
 * @returns {function}
 */
'use strict'

const co = require('co')
const errors = require('../errors')
const successPayload = require('apeman-app-rest/lib/payloads/success_payload')
const failurePayload = require('apeman-app-rest/lib/payloads/failure_payload')
const apemanmodel = require('apemanmodel')

/** @lends doUpdateRecover */
function doUpdateRecover (config) {
  const ACTION_NAME = 'update'
  let success = successPayload(ACTION_NAME)
  let failure = failurePayload(ACTION_NAME)

  let digest = config.digest
  return function updateRecover (UserRecover, context) {
    let { data, id, transaction } = context
    const User = UserRecover.getRef('user').model
    const UserSign = User.getRef('sign').model
    let one = apemanmodel.defineOne(UserRecover, transaction)
    let update = apemanmodel.defineUpdate(UserRecover, transaction)

    let type = apemanmodel.toType(UserRecover)
    return co(function * () {
      let recover = yield one(id, [])
      if (!recover) {
        return failure(errors.notFoundError(type, { id }))
      }
      if (recover.token !== String(data.attributes.token)) {
        return failure(errors.tokenInvalidError())
      }
      if (recover.expired) {
        return failure(errors.tokenExpiredError())
      }
      if (recover.done) {
        return failure(errors.tokenExpiredError())
      }
      let userId = recover.userId
      let signs = yield UserSign.findAll({
        where: { userId }
      })
      for (let sign of signs) {
        let hash = yield digest(data.attributes.password || '', sign.hashSalt)
        yield update(sign, {
          hash: hash,
          signmodAt: new Date()
        })
      }
      let updated = yield update(recover, {
        done: true,
        doneAt: new Date()
      })
      return success(
        {},
        apemanmodel.toData(UserRecover, updated),
        {}
      )
    })
  }
}

module.exports = doUpdateRecover
